//  Pac Man I
//
//  Copyright (c) Cyrion Software
//  All Rights Reserved
//
//  +-- button.cc: (all functions related to buttons) ---------------------+
//
//  Last Updated: 11 of April of 1999 <rp>
//  Tools       : DJGPP v2 + Allegro 3.1 + RHIDE 1.4
//  Target      : DOS v5.0+


#include "pacgame.h"


/* +------------------------------------------------------- DrawFrame ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Desenha as bordas dos botes com linha dupla e sombra    |
   +---------------------------------------------------------------------+ */

void DrawFrame(int x1, int y1, int x2, int y2, int col1, int col2)
{
                                                                   // de fora
  hline(screen, x1, y1, x2, col1);                               // - de cima
  hline(screen, x1, y2, x2, col2);                              // - de baixo
  vline(screen, x1, y1, y2, col1);                      // | do lado esquerdo
  vline(screen, x2, y1, y2, col2);                       // | do lado direito
                                                                 // de dentro
  hline(screen, x1+1, y1+1, x2-1, col1);                         // - de cima 
  hline(screen, x1+1, y2-1, x2-1, col2);                        // - de baixo
  vline(screen, x1+1, y1+1, y2-1, col1);                // | do lado esquerdo
  vline(screen, x2-1, y1+1, y2-1, col2);                 // | do lado direito
}

/* +------------------------------------------------ fim de DrawFrame ---+ */

/* +-------------------------------------------------------- Dtextout ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Coloca o texto, com sombra, nos botes                   |
   +---------------------------------------------------------------------+ */

void Dtextout(BITMAP* s, FONT* f, char *str, int x1, int y1, int col1, int col2)
{
  text_mode(-1);                 // o texto vai ser escrito transparentemente
  textout(s, f, str, x1+1, y1+1, col2);                       // faz a sombra
  textout(s, f, str, x1, y1, col1);                    // faz a parte de cima
}

/* +------------------------------------------------- fim de Dtextout ---+ */

/* +------------------------------------------------- Dtextout_centre ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Coloca o texto, com sombra, nos botes mas centrado      |
   +---------------------------------------------------------------------+ */

void Dtextout_centre(BITMAP *s, FONT *f, char *str, int x1, int y1, int col1, int col2)
{
  text_mode(-1);                 // o texto vai ser escrito transparentemente
  textout_centre(s, f, str, x1+1, y1+1, col2);                // faz a sombra
  textout_centre(s, f, str, x1, y1, col1);             // faz a parte de cima
}

/* +------------------------------------------ fim de Dtextout_centre ---+ */

/* +------------------------------------------------------ mouseinbox ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Verifica se o rato se encontra nas coordenadas dadas     |
   +---------------------------------------------------------------------+ */

bool mouseinbox(int x1, int y1, int x2, int y2)
{
  if ((mouse_x > x1) && (mouse_x < x2)) {
    if ((mouse_y > y1) && (mouse_y < y2))
      return TRUE;                                                // encontra
  }
  return FALSE;                                            // no se encontra
}

/* +----------------------------------------------- fim de mouseinbox ---+ */

/* +---------------------------------------------------------- Button ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Funo principal na construco dos botes               |
   +---------------------------------------------------------------------+ */

Button::Button(char *n, int xa, int ya, int xb, int yb, int f1, int f2, BITMAP *btn)
{
  x1 = xa;
  x2 = xb;
  y1 = ya;
  y2 = yb;
  frame1 = f1;
  frame2 = f2;
  bitmap = btn;
  pressed = FALSE;
  if (n)
    strcpy(name, n);
  else
    name[0] = NULL;
}

/* +--------------------------------------------------- fim de Button ---+ */

/* +------------------------------------------------------ ChangeName ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Muda o texto actual do boto para o indicado             |
   +---------------------------------------------------------------------+ */

void Button::ChangeName(char *n)
{
  strcpy(name, n);
}

/* +----------------------------------------------- fim de ChangeName ---+ */

/* +------------------------------------------------------ DrawButton ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Desenha o boto                                          |
   +---------------------------------------------------------------------+ */

void Button::DrawButton(void)
{
  show_mouse(NULL);                                         // esconde o rato
  if (name[0] == NULL)                                     // se no tem nome
    blit(bitmap, screen, 0, 0, x1, y1, x2-x1, y2-y1);
  else                                                         // se tem nome
  {
    drawing_mode(DRAW_MODE_COPY_PATTERN, bitmap, 0, 0);
    rectfill(screen, x1, y1, x2, y2, 1);
  }
  solid_mode();                                     // modo slido de escrita
  int temp = y1 +((y2 - y1) - text_height((FONT*)graphics[FONT_NORMAL].dat)) / 2;
  if (pressed) {                // se o boto foi pressionado, pressed = TRUE
    DrawFrame(x1, y1, x2, y2, frame1, frame2);
    if (name)                                 // se tem nome, name[0] <> NULL
      Dtextout_centre(screen, (FONT*)graphics[FONT_NORMAL].dat, name, x1 +(x2 - x1) / 2, temp, frame1, frame2);
  }
  else                     // se o boto no foi pressionado, pressed = FALSE
  {
    DrawFrame(x1, y1, x2, y2, frame2, frame1);
    if (name)                                                  // se tem nome
      Dtextout_centre(screen, (FONT*)graphics[FONT_NORMAL].dat,name, x1+(x2-x1)/2, temp, frame2, frame1);
  }
  show_mouse(screen);                                        // mostra o rato
}

/* +----------------------------------------------- fim de DrawButton ---+ */

/* +--------------------------------------------------------- Clicked ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Verifica se o boto foi clicado                          |
   +---------------------------------------------------------------------+ */

bool Button::Clicked(void)
{
  if ((mouseinbox(x1, y1, x2, y2)) && (mouse_b & 1)) { 
    pressed = TRUE;
    DrawButton();
    if (sound_on)                                               // se h sons
      play_sample((SAMPLE*)sounds[BUTTON_CLICK].dat, sound_volume, 125, 1000, FALSE);
    do {
      if ((mouseinbox(x1, y1, x2, y2)) && (pressed == FALSE)) {
        if (sound_on)                                           // se h sons
          play_sample((SAMPLE*)sounds[BUTTON_CLICK].dat, sound_volume, 125, 1000, FALSE);
        pressed = TRUE;
        DrawButton();                                      // desenha o boto
      }
      else            
        if (!(mouseinbox(x1, y1, x2, y2)) && (pressed == TRUE)) {
          pressed = FALSE;
          DrawButton();                                    // desenha o boto
        }
    } while (mouse_b & 1);
    int temp = pressed;
    pressed = FALSE;
    DrawButton();                                          // desenha o boto
    return temp;
  }
  return FALSE;
}

/* +-------------------------------------------------- end of Clicked ---+ */


/* +------------------------------------------------ end of button.cc ---+ */
