//  Pac Man I
//
//  Copyright (c) Cyrion Software
//  All Rights Reserved
//
//  +-- game.cc: (all functions related to the game) --------------------+
//
//  Last Updated: 11 of April of 1999 <rp>
//  Tools       : DJGPP v2 + Allegro 3.1 + RHIDE 1.4
//  Target      : DOS v5.0+

                                                // declarao de header files
#include "pacgame.h"


#define SCORE_X   500
#define SCORE_Y   210

struct player_struct {                                // estrutura do jogador
   int x_loc, y_loc;                                        // usado pelo pac
   int x_dir, y_dir;                                        // e os fantasmas
   int state;
   int frame;
};

struct map_struct *gmap;

struct player_struct eaters[20];
struct player_struct pac;

int mapV[22][22];
int mapBlood[22][22];

volatile int pac_timer        = 0;
volatile int eater_timer      = 0;
volatile int eater_dead_timer = 0;
volatile int ball_timer       = 0;
volatile int frame_rate_timer = 0;

int pac_speed, eater_speed;
int state_timer        = 0;
int eater_move_counter = 0;
int pac_move_counter   = 0;
int pac_move_direction = 0;        // 0-nenhum, 1-cima, 2-baixo, 3-esq, 4-dir

int current_map_played;
int score;
int num_balls;
int lives;
int end_game;
int eaters_eaten;
int max_eaters_state;
int num_eaters;
int extra_life;

struct HighScoresStruct HighScores[MAX_HIGHSCORES];

                                                  // declarao de prottipos
void MoveEater(int which);
void ChangeEaterDir(int which);
int  EaterGoUp(int which);
int  EaterGoLeft(int which);
int  EaterGoDown(int which);
int  EaterGoRight(int which);
void MoveLeft(void);
void MoveRight(void);
void MoveDown(void);
void MoveUp(void);
void Blood(void);
void GetHighScore(void);
void GotHighScore(void);


/* +---------------------------------------------------------- Random ---+
   | Verso   : 1.00.00                                                  |
   | Funo   : Gera nmeros aleatrios                                  |
   +---------------------------------------------------------------------+ */

int Random(int x) {     
  return random() % x;
}

/* +--------------------------------------------------- fim de Random ---+ */

/* +------------------------------------------------- increment_timer ---+
   | Verso   : 1.00.00                                                  |
   | Funo   : Incrementa o timer                                       |
   +---------------------------------------------------------------------+ */

void increment_timer(...)
{
  pac_timer++;
  eater_timer++;
  eater_dead_timer++;
  ball_timer++;
  frame_rate_timer++;
}
END_OF_FUNCTION(increment_timer);

/* +------------------------------------------ fim de increment_timer ---+ */

/* +------------------------------------------------------- PlaySound ---+
   | Verso   : 1.00.00                                                  |
   | Funo   : Toca som especificado                                    |
   +---------------------------------------------------------------------+ */

void PlaySound(int which)
{
  play_sample((SAMPLE*)sounds[which].dat, sound_volume, 125, 1000, FALSE);
}

/* +------------------------------------------------ fim de PlaySound ---+ */

/* +------------------------------------------------ InitGameGraphics ---+
   | Verso   : 1.00.00                                                  |
   | Funo   : Inicializa os grficos do jogo                           |
   +---------------------------------------------------------------------+ */

void InitGameGraphics(void)
{
  clear(screen);
  draw_sprite(screen, (BITMAP*)graphics[SCORE_BOX].dat, 465, 160);
}

/* +----------------------------------------- fim de InitGameGraphics ---+ */

/* +----------------------------------------------------- DrawGameMap ---+
   | Verso   : 1.00.00                                                  |
   | Funo   : Desenha o mapa                                           |
   +---------------------------------------------------------------------+ */

void DrawGameMap(void)
{
  int i, j;
  clear(map_bitmap);
  for (i = 0; i < 22; i++) {
    for (j = 0; j < 22; j++) {
      if (gmap->values[i][j] < 700)
        draw_sprite(map_bitmap, (BITMAP*)set_graphics[gmap->values[i][j]/100][gmap->values[i][j]%100].dat, i*20, j*20);
      if (mapV[i][j] > 0 && mapV[i][j] < 9)
        draw_sprite(map_bitmap, (BITMAP*)set_graphics[7][mapV[i][j]].dat, i*20, j*20);
      if (mapBlood[i][j] == 2)                             // coloca o sangue
        draw_sprite(map_bitmap, (BITMAP*)sprites[BLOOD_RIGHT].dat, i*20, j*20);
      else
        if (mapBlood[i][j] == 4)                           // coloca o sangue
          draw_sprite(map_bitmap, (BITMAP*)sprites[BLOOD_DOWN].dat, i*20,j*20);
        else
          if (mapBlood[i][j] == 8)                         // coloca o sangue
            draw_sprite(map_bitmap, (BITMAP*)sprites[BLOOD_LEFT].dat, i*20, j*20);
          else
            if (mapBlood[i][j] == 16)                      // coloca o sangue
              draw_sprite(map_bitmap, (BITMAP*)sprites[BLOOD_UP].dat, i*20, j*20);
    }
  }
  for (i = 0; i < num_eaters; i++)
    draw_sprite(map_bitmap, (BITMAP*)sprites[eaters[i].frame].dat, eaters[i].x_loc, eaters[i].y_loc);
  draw_sprite(map_bitmap, (BITMAP*)sprites[pac.frame].dat, pac.x_loc, pac.y_loc);
  blit(map_bitmap, screen, 0, 0, 20, 20, 440, 440);
  sprintf(text, "Score : %06d", score);
  text_mode(159);
  textout(screen, (FONT*)graphics[FONT_NORMAL].dat, text, SCORE_X, SCORE_Y, 32);
}

/* +---------------------------------------------- fim de DrawGameMap ---+ */

/* +------------------------------------------------------ StartLevel ---+
   | Verso   : 1.00.00                                                  |
   | Funo   : Inicia o nvel                                           |
   +---------------------------------------------------------------------+ */

void StartLevel(void)
{
  for (int i = 0; i < num_eaters; i++) {
    eaters[i].x_loc = gmap->eater_x * 20;
    eaters[i].y_loc = gmap->eater_y * 20;
    eaters[i].x_dir = 5;
    eaters[i].y_dir = 0;
    eaters[i].frame = EATER01;
    eaters[i].state = 0;
  }
  pac.x_loc = gmap->pac_x * 20;
  pac.y_loc = gmap->pac_y * 20;
  pac.frame = PAC1A;
  text_mode(159);
  sprintf(text, "Lives : %6d", lives);
  textout(screen, (FONT*)graphics[FONT_NORMAL].dat, text, SCORE_X, SCORE_Y + 40, 15);
  sprintf(text, "Level : %6d", current_map_played+1);
  textout(screen, (FONT*)graphics[FONT_NORMAL].dat, text, SCORE_X, SCORE_Y + 75, 15);
  textout(screen, (FONT*)graphics[FONT_NORMAL].dat, "High Score:", SCORE_X, SCORE_Y + 100, 15);
  textout(screen, (FONT*)graphics[FONT_NORMAL].dat, HighScores[0].name, SCORE_X, SCORE_Y+120, 15);
  sprintf(text, "%06d", HighScores[0].score);
  textout(screen, (FONT*)graphics[FONT_NORMAL].dat, text, SCORE_X, SCORE_Y + 140, 15);
  PlaySong();                                         // toca msica de fundo
}

/* +----------------------------------------------- fim de StartLevel ---+ */

/* +-------------------------------------------------- PlayCustomGame ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Controla o jogo num mapa especificado pelo utilizador    |
   +---------------------------------------------------------------------+ */

void PlayCustomGame(void)
{
  FILE *file;
  text[0] = '\0';
  gui_fg_color = 15;
  gui_bg_color = 1;
  if (file_select("Select Custom Map", text, "map") != 0) {
    file = fopen(text, "rb");
    if (!file)
      alert("Error opening file.", NULL, NULL, "OK", NULL, KEY_ENTER, 0);
    else {
      fread(&map, sizeof(struct map_struct), 1, file);
      fclose(file);
      show_mouse(NULL);                                     // esconde o rato
      fade_out(3);
      clear(screen);
      clear_keybuf();                            // limpa o buffer do teclado
      InitGameGraphics();
      fade_in((RGB*)graphics[PAL_EDIT].dat, 200);
      score = 0;
      if (dif_level == 0) {
        lives = 5;
        max_eaters_state = 300;
        pac_speed = 7;
        eater_speed = 14;
      }
      else
        if (dif_level == 1) {
          lives = 4;
          max_eaters_state = 200;
          pac_speed = eater_speed = 10;
        }
        else
        if (dif_level == 2) {
          lives = 3;
          max_eaters_state = 100;
          pac_speed = 12;
          eater_speed = 7;
        }
      extra_life = 0;
      current_map_played = 0;
      PlayMap(&map);                      // jogar apenas o mapa especificado
      GotHighScore();
      show_mouse(screen);                                    // mostra o rato
    }
  }
}

/* +------------------------------------------- fim de PlayCustomGame ---+ */

/* +--------------------------------------------------------- PlayMap ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Controla o jogo num determinado mapa                     |
   +---------------------------------------------------------------------+ */

void PlayMap(struct map_struct *map)
{
  int i, j;
  gmap = map;
  pac_timer = eater_timer = ball_timer = eater_dead_timer = 0;
  LOCK_VARIABLE(pac_timer);
  LOCK_VARIABLE(eater_timer);
  LOCK_VARIABLE(ball_timer);
  LOCK_VARIABLE(eater_dead_timer);
  LOCK_FUNCTION(increment_timer);
  install_int(increment_timer, 5);
  eater_move_counter = 0;
  num_balls = gmap->num_balls;
  end_game = FALSE;
  eaters_eaten = 0;
  num_eaters = map->num_eaters+dif_level;
  if (num_eaters > 20)
    num_eaters = 20;
  StartLevel();
  for (i = 0; i < 22; i++) {
    for (j = 0; j < 22; j++) {
      if (map->values[i][j] < 700)
        mapV[i][j] = 0;
      else
        if (map->values[i][j] < 709)
          mapV[i][j] = map->values[i][j]-700;
        else
          mapV[i][j] = -1;
      mapBlood[i][j] = 0;
    }
  }
  if (current_map_played >= MAX_MAPS)
    end_game = TRUE;
  while (end_game != TRUE) {                         // inicia o loop do jogo
    if (key[KEY_ESC]) {                                                // ESC
      while (key[KEY_ESC]) { }
      text_mode(-1);
      textout(screen, (FONT*)graphics[FONT_NORMAL].dat,"Are you sure you want to quit? (Y/N)", 100, 200, 15);
      clear_keybuf();                            // limpa o buffer do teclado
      int ch = readkey();
      if (ch >> 8 == KEY_Y) {                                            // Y
        end_game = TRUE;
        break;
      }
    } 
    else
      if (key[KEY_F10]) {                  // F10 - grava cran para ficheiro
        while (key[KEY_F10]) { }
        BITMAP* bmp;
        PALLETE pal;
        bmp = create_sub_bitmap(screen, 0, 0, SCREEN_W, SCREEN_H);
        get_pallete(pal);
        save_pcx("pacgame.pcx", bmp, pal);
        destroy_bitmap(bmp);
      }
    poll_joystick();
    if (pac_timer > pac_speed) {
      if (!pac_move_direction) {
        if (key[KEY_UP] | joy_up)                                // Para cima
          MoveUp();
        if (key[KEY_DOWN] | joy_down)                           // Para baixo
          MoveDown();
        if (key[KEY_LEFT] | joy_left)                      // Para a esquerda
          MoveLeft();
        if (key[KEY_RIGHT] | joy_right)                     // Para a direita
          MoveRight();       
      }
      else {
        if (pac_move_direction == 1)
          MoveUp();
        if (pac_move_direction == 2)
          MoveDown();
        if (pac_move_direction == 3)
          MoveLeft();
        if (pac_move_direction == 4)
          MoveRight();
      }
      pac_timer = 0;
    }
    if (eater_timer > eater_speed) {
      for (i = 0; i < num_eaters; i++)             // move todos os fantasmas
        MoveEater(i);
        eater_move_counter++;
        eater_timer = 0;
    }
    if (state_timer != 0) {                        // se state_timer activado
      state_timer++;                                          // incrementa-o
      if (state_timer > max_eaters_state) { // se state time = 100 = fantasma normal
        for (i = 0; i < num_eaters; i++)
          eaters[i].state = 0;
        state_timer = 0;                             // desactiva state_timer
        eaters_eaten = 0;
      }
    }
    DrawGameMap();                                          // desenha o mapa
                                                          // verifica coliso
    for (i = 0; i < num_eaters; i++) {
      if ((pac.x_loc >= eaters[i].x_loc && pac.x_loc < eaters[i].x_loc + 20) || (pac.x_loc + 20 > eaters[i].x_loc && pac.x_loc + 20 < eaters[i].x_loc + 20)) {
        if ((pac.y_loc >= eaters[i].y_loc && pac.y_loc < eaters[i].y_loc + 20) || (pac.y_loc + 20 > eaters[i].y_loc && pac.y_loc + 20 < eaters[i].y_loc + 20)) {
          if (eaters[i].state == 2) {  // se estado do fantasma = 2 = comilo pode ser comido
            if (sound_on)                                        // se h som
              PlaySound(EATEATER);
            eaters[i].x_loc = gmap->eater_x * 20;
            eaters[i].y_loc = gmap->eater_y * 20;
            eaters[i].state = 0;
            eaters_eaten++;
            score += 200 * eaters_eaten + 200 * dif_level;
            Blood();                           // coloca o sangue nas paredes
          }
          else {                                                 // pac morre
            Blood();                           // coloca o sangue nas paredes
            if (lives > 1) {
              if (sound_on)
                PlaySound(PACDIE);                               // se h som
              lives--;                               // reduz nmero de vidas
              text_mode(-1);
              textout(screen, (FONT*)graphics[FONT_GAME].dat, "You died!!!", 200, 240, 15);
              textout(screen, (FONT*)graphics[FONT_NORMAL].dat, "Press <space> ", 200, 265, 15);
              while (!key[KEY_SPACE]);
              StartLevel();
            }
            else {                                         // j no h vidas
              end_game = TRUE;
              if (sound_on)                                      // se h som
                PlaySound(GAMEOVER_SOUND);
            }
          }
        }
      }
    }
    if (num_balls == 0) {                             // se no h mais bolas
      if (sound_on)                                              // se h som
        PlaySound(ENDLEVEL);
      break;
    }
    if (end_game == TRUE)
      break;
    if (score - extra_life > 20000 + 15000 * dif_level) {
      if (sound_on)                                              // se h som
        PlaySound(EXTRA_LIFE);
      extra_life = score;
      lives++;
      text_mode(159);
      sprintf(text, "Lives : %6d", lives);
      textout(screen, (FONT*)graphics[FONT_NORMAL].dat, text, SCORE_X, SCORE_Y + 40, 15);
    }
  };
  remove_int(increment_timer);
  stop_midi();                                      // para a msica de fundo
  if (end_game == TRUE) {
    draw_sprite(screen, (BITMAP*)graphics[GAME_OVER].dat, 150, 160);
    while (!key[KEY_SPACE]);
  }
}

/* +-------------------------------------------------- fim de PlayMap ---+ */

/* +---------------------------------------------------- GotHighScore ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Obtm o "high score"                                     |
   +---------------------------------------------------------------------+ */

void GotHighScore(void)
{
  if (score > HighScores[MAX_HIGHSCORES-1].score) {
    if (sound_on) PlaySound(HIGH_SCORE);
      int i;
    for (i = MAX_HIGHSCORES-1; i >= 0; i--) {
      if (score > HighScores[i].score) {
        if (i > 0) {
          HighScores[i].score = HighScores[i-1].score;
          HighScores[i].level = HighScores[i-1].level;
          strcpy(HighScores[i].name, HighScores[i-1].name);
        } 
      }
      else
        break;
    }
    HighScores[i+1].score = score;
    HighScores[i+1].level = current_map_played;
    GetHighScore();
    strcpy(HighScores[i+1].name, text);
    ViewHighScores();
  }
}

/* +--------------------------------------------- fim de GotHighScore ---+ */

/* +-------------------------------------------------------- PlayGame ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Jogar!!                                                  |
   +---------------------------------------------------------------------+ */

void PlayGame(void)
{
  current_map_played = 0;

  if (dif_level == 0) {                                              // fcil
    lives = 5;
    max_eaters_state = 300;
    pac_speed = 7;
    eater_speed = 14;
  }
  else
    if (dif_level == 1) {                                           // normal
      lives = 4;
      max_eaters_state = 200;
      pac_speed = eater_speed = 10;
    } 
    else
      if (dif_level == 2) {                                        // difcil
     	lives = 3;
        max_eaters_state = 100;
        pac_speed = 12;
        eater_speed = 7;
      }
      else
        Error("Pac Man I: wrong number of lives.\n");
  extra_life = 0;
  show_mouse(NULL);                                         // esconde o rato
  fade_out(10);
  clear_keybuf();                                // limpa o buffer do teclado
  InitGameGraphics();
  fade_in((RGB*)graphics[PAL_EDIT].dat, 200);
  score = 0;
  do {
    PlayMap((struct map_struct*)mapData[current_map_played].dat);
    current_map_played++;
    if (current_map_played >= MAX_MAPS) {     // fim do jogo, no h + nveis 
      end_game = TRUE;
    }
  } while (end_game != TRUE);
  GotHighScore();                          // obtm o "high score", se houver
  show_mouse(screen);                                        // mostra o rato
  stop_midi();                                      // para a msica de fundo
}

/* +------------------------------------------------- fim de PlayGame ---+ */

/* +------------------------------------------------------- MoveEater ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Move comilo                                             |
   +---------------------------------------------------------------------+ */

void MoveEater(int which)
{
  if (eater_move_counter % 8 >= 0 || eater_move_counter % 8 <= 4)
    eaters[which].frame = EATER01;
  else
    eaters[which].frame = EATER02;
                                   // se o estado  o 2 o fantasma  vermelho
  if (eaters[which].state == 2)
    eaters[which].frame = EATER03;

             // se o comilo est num quadrado perfeito pode mudar a direco
  if ((eaters[which].x_loc % 20 == 0) && (eaters[which].y_loc % 20 == 0))
    ChangeEaterDir(which);
                                          // incrementa a posio do fantasma
  eaters[which].x_loc += eaters[which].x_dir;
  eaters[which].y_loc += eaters[which].y_dir;
}

/* +------------------------------------------------ fim de MoveEater ---+ */

/* +-------------------------------------------------- ChangeEaterDir ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Muda a direco do fantasma se possvel                  |
   +---------------------------------------------------------------------+ */

void ChangeEaterDir(int which)
{
  int dir = Random(4-dif_level);

  if (dir == 1 && eaters[which].state != 2) {
    if (eaters[which].x_dir == 5) {
      if (eaters[which].x_loc < pac.x_loc && EaterGoRight(which))
        return;
      else
        if (eaters[which].y_loc < pac.y_loc && EaterGoDown(which))
          return;
        else
          if (eaters[which].y_loc > pac.y_loc && EaterGoUp(which))
            return;
    }
    else
      if (eaters[which].x_dir == -5) {
        if (eaters[which].x_loc > pac.x_loc && EaterGoLeft(which))
          return;
        else
          if (eaters[which].y_loc < pac.y_loc && EaterGoDown(which))
            return;
          else
            if (eaters[which].y_loc > pac.y_loc && EaterGoUp(which))
              return;
      }
      else
        if (eaters[which].y_dir == 5) {
          if (eaters[which].y_loc < pac.y_loc && EaterGoDown(which))
            return;
          else
            if (eaters[which].x_loc < pac.x_loc && EaterGoRight(which))
              return;
            else
              if (eaters[which].x_loc > pac.x_loc && EaterGoLeft(which))
                return;
        }
   	else
          if (eaters[which].y_dir == -5) {
            if (eaters[which].y_loc > pac.y_loc && EaterGoUp(which))
              return;
            else
              if (eaters[which].x_loc < pac.x_loc && EaterGoRight(which))
                return;
              else
                if (eaters[which].x_loc > pac.x_loc && EaterGoLeft(which))
                  return;
          }
  }
  dir = Random(3);
  if (eaters[which].x_dir == 5 || eaters[which].x_dir == -5) {  // esq ou dta
    if (dir == 2) {   
      if (EaterGoDown(which))
        return;
      else
        dir = Random(2);
    }
    if (dir == 1) {
      if (EaterGoUp(which))
        return;
    }
    if (eaters[which].x_dir == 5) {
      if (EaterGoRight(which))
        return;
      else
        EaterGoLeft(which);
      return;
    }
    else {
      if (EaterGoLeft(which))
        return;
      else
        EaterGoRight(which);
      return;
    }
  }
  if (eaters[which].y_dir == 5 || eaters[which].y_dir == -5) { // bxo ou cma
    if (dir == 2) {
      if (EaterGoLeft(which))
        return;
      else
        dir = Random(2);
    }
    if (dir == 1) {
      if (EaterGoRight(which))
        return;
    }
    if (eaters[which].y_dir == 5) {
      if (EaterGoDown(which))
        return;
      else
        EaterGoUp(which);
      return;
    }
    else {
      if (EaterGoUp(which))
        return;
      else
        EaterGoDown(which);
      return;
    }
  }
}

/* +------------------------------------------- fim de ChangeEaterDir ---+ */

/* +------------------------------------------------------- EaterGoUp ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Move o comilo para cima                                 |
   +---------------------------------------------------------------------+ */

int EaterGoUp(int which)
{
  if (mapV[eaters[which].x_loc / 20][eaters[which].y_loc / 20 -1] == 0)
    return FALSE;
  eaters[which].x_dir = 0;
  eaters[which].y_dir = -5;
  if (eaters[which].y_loc == 5) eaters[which].y_dir = 5; 
    return TRUE;
}

/* +------------------------------------------------ fim de EaterGoUp ---+ */

/* +----------------------------------------------------- EaterGoDown ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Move o comilo para baixo                                |
   +---------------------------------------------------------------------+ */

int EaterGoDown(int which)
{
  if (mapV[eaters[which].x_loc / 20][eaters[which].y_loc / 20 +1] == 0)
    return FALSE;
  eaters[which].x_dir = 0;
  eaters[which].y_dir = 5;
  if (eaters[which].y_loc == 415) eaters[which].y_dir = -5;
    return TRUE;
}

/* +---------------------------------------------- fim de EaterGoDown ---+ */

/* +---------------------------------------------------- EaterGoRight ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Move o comilo para a direita                            |
   +---------------------------------------------------------------------+ */

int EaterGoRight(int which)
{
  if (mapV[eaters[which].x_loc / 20+1][eaters[which].y_loc / 20] == 0)
    return FALSE;
  eaters[which].x_dir = 5;
  eaters[which].y_dir = 0;
  if (eaters[which].x_loc >= 415) eaters[which].x_dir = -5;
    return TRUE;
}

/* +--------------------------------------------- fim de EaterGoRight ---+ */

/* +----------------------------------------------------- EaterGoLeft ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Move o comilo para a esquerda                           |
   +---------------------------------------------------------------------+ */

int EaterGoLeft(int which)
{
  if (mapV[eaters[which].x_loc / 20-1][eaters[which].y_loc / 20] == 0)
    return FALSE;
  eaters[which].x_dir = -5;
  eaters[which].y_dir = 0;
  if (eaters[which].x_loc == 5) eaters[which].x_dir = 5;
    return TRUE;
}

/* +---------------------------------------------- fim de EaterGoLeft ---+ */

/* +--------------------------------------------------------- EatBall ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Verifica se o pac comeu a bola                           |
   +---------------------------------------------------------------------+ */

void EatBall(void)
{
  int temp = mapV[pac.x_loc / 20][pac.y_loc / 20];
  if (temp > 0 && temp < 5) {                                // bola pequena?
    mapV[pac.x_loc / 20][pac.y_loc / 20] = -1;           // j no h bola a      
    num_balls--;                                // decrementa nmero de bolas
    if (sound_on)                                                // se h som
      PlaySound(EATSBALL);
    score += gmap->scores[temp-1] + 10 * dif_level;        // aumenta o score
  }
  else
    if (temp > 4 && temp < 9) {                               // bola grande?
      mapV[pac.x_loc / 20][pac.y_loc / 20] = -1;         // j no h bola a
      state_timer = 1;                                      // activa o timer 
      score += gmap->scores[temp-1] + 100 * dif_level;     // aumenta o score
      if (sound_on)                                              // se h som
        PlaySound(EATBBALL);
      for (int i = 0; i < num_eaters; i++)
        eaters[i].state = 2;              // todos os fantasmas para estado 2
    }
}

/* +-------------------------------------------------- fim de EatBall ---+ */

/* +---------------------------------------------------------- MoveUp ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Move para cima                                           |
   +---------------------------------------------------------------------+ */

void MoveUp(void)
{
  if (pac.x_loc % 20 != 0)
    return; 
  pac_move_direction = 1;
  if (pac.y_loc % 20 == 0) {
    if (mapV[pac.x_loc / 20][pac.y_loc / 20 -1] == 0) {
      pac_move_direction = 0;
      return;
    }
  }
  if (pac.y_loc % 10 == 0)
    pac.frame = PAC3A;
  else
    pac.frame = PAC3B;
  EatBall();                                                     // come bola
  pac.y_loc -= 5;                                            // muda posio
  if (pac.y_loc % 20 == 0)
    pac_move_direction = 0;
  if (pac.y_loc < 5)
    pac.y_loc = 415;
}

/* +--------------------------------------------------- fim de MoveUp ---+ */

/* +-------------------------------------------------------- MoveDown ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Move para baixo                                          |
   +---------------------------------------------------------------------+ */

void MoveDown(void)
{
  if (pac.x_loc % 20 != 0)
    return;
  pac_move_direction = 2;
  if (pac.y_loc % 20 == 0){
    if (mapV[pac.x_loc / 20][pac.y_loc / 20 + 1] == 0) {
      pac_move_direction = 0;
      return;
    }
  }
  if (pac.y_loc % 10 == 0)
    pac.frame = PAC4A;
  else
    pac.frame = PAC4B;
  EatBall();                                                     // come bola
  pac.y_loc += 5;
  if (pac.y_loc % 20 == 0)
    pac_move_direction = 0;
  if (pac.y_loc > 415)
    pac.y_loc = 5;
}

/* +------------------------------------------------- fim de MoveDown ---+ */

/* +------------------------------------------------------- MoveRight ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Move para a direita                                      |
   +---------------------------------------------------------------------+ */

void MoveRight(void)
{
  if (pac.y_loc % 20 != 0)
    return;
  pac_move_direction = 4;
  if (pac.x_loc % 20 == 0) {
    if (mapV[pac.x_loc / 20 +1][pac.y_loc / 20] == 0) {
      pac_move_direction = 0;
      return;
    }
  }
  if (pac.x_loc % 10 == 0)
    pac.frame = PAC1A;
  else
    pac.frame = PAC1B;
  EatBall();                                                     // come bola
  pac.x_loc += 5;
  if (pac.x_loc % 20 == 0)
    pac_move_direction = 0;
  if (pac.x_loc > 415)
    pac.x_loc = 5;
}

/* +------------------------------------------------ fim de MoveRight ---+ */

/* +-------------------------------------------------------- MoveLeft ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Move para a esquerda                                     |
   +---------------------------------------------------------------------+ */

void MoveLeft(void)
{
  if (pac.y_loc % 20 != 0)
    return;
  pac_move_direction = 3;
  if (pac.x_loc % 20 == 0) {
    if (mapV[pac.x_loc / 20 -1][pac.y_loc / 20] == 0) {
      pac_move_direction = 0;
      return;
    }
  }
  if (pac.x_loc % 10 == 0)
    pac.frame = PAC2A;
  else
    pac.frame = PAC2B;
  EatBall();                                                     // come bola
  pac.x_loc -= 5;
  if (pac.x_loc % 20 == 0)
    pac_move_direction = 0;
  if (pac.x_loc < 5)
    pac.x_loc = 415;
}

/* +------------------------------------------------- fim de MoveLeft ---+ */

/* +----------------------------------------------------------- Blood ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Mostra sangue nas paredes quando algum morre            |
   +---------------------------------------------------------------------+ */

void Blood(void)
{
  if (mapV[pac.x_loc / 20+1][pac.y_loc / 20] == 0)
    mapBlood[pac.x_loc / 20+1][pac.y_loc / 20] = 8;
  if (mapV[pac.x_loc / 20-1][pac.y_loc / 20] == 0)
    mapBlood[pac.x_loc / 20-1][pac.y_loc / 20] = 2;
  if (mapV[pac.x_loc / 20][pac.y_loc / 20-1] == 0)
    mapBlood[pac.x_loc / 20][pac.y_loc / 20-1] = 4;
  if (mapV[pac.x_loc / 20][pac.y_loc / 20+1] == 0)
    mapBlood[pac.x_loc / 20][pac.y_loc / 20+1] = 16;
}

/* +---------------------------------------------------- fim de Blood ---+ */

/* +---------------------------------------------------- GetHighScore ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Obtm o "high score"                                     |
   +---------------------------------------------------------------------+ */

void GetHighScore(void)
{
  drawing_mode(DRAW_MODE_COPY_PATTERN, (BITMAP*)graphics[PATTERN2].dat, 0, 0);
  rectfill(screen, 100, 150, 340, 280, 1);
  solid_mode();
  text_mode(-1);
  rect(screen, 100,150, 340, 280, 25);
  textout_centre(screen, (FONT*)graphics[FONT_GAME].dat, "High Score", 220, 160, 15);
  rectfill(screen, 150, 200, 290, 240, 0);
  rect(screen, 150, 200, 290, 240, 25);
  int l = 0;
  int ch;
  text[l] = '\0';
  clear_keybuf();                                // limpa o buffer do teclado
  text_mode(0);
  textout(screen, (FONT*)graphics[FONT_NORMAL].dat, "_", 155, 212, 15);
  do {
    if (keypressed()) {
      ch = readkey() & 0x00ff;
      if ((ch > 31) && (ch < 127) && (l < 14)) {
        text[l+1] = '\0';
        text[l] = ch;
        l++;
        PlaySound(TYPE_LETTER);                                // toca um som
      }
      else
        if ((key[KEY_BACKSPACE]) && (l>0)) {                // <--- Backspace
            PlaySound(TYPE_LETTER);                            // toca um som
            l--;
            text[l] = '\0';
            rectfill(screen, 151, 201, 289, 239, 0);
        }
      	else
          if (key[KEY_ENTER])                                  //  <--| Enter
            break;                                       // acaba a digitao
      textout(screen, (FONT*)graphics[FONT_NORMAL].dat, text, 155, 212, 15);
      textout(screen, (FONT*)graphics[FONT_NORMAL].dat, "_", text_length((FONT*)graphics[FONT_NORMAL].dat, text)+155, 212, 15);
    }
  } while(1==1);
  text_mode(-1);
}

/* +--------------------------------------------- fim de GetHighScore ---+ */


/* +-------------------------------------------------- fim de game.cc ---+ */
