//  Pac Man I
//
//  Copyright (c) Cyrion Software
//  All Rights Reserved
//
//  +-- main.cc: (controls the game ) -----------------------------------+
//
//  Last Updated: 11 of April of 1999 <rp>
//  Tools       : DJGPP v2 + Allegro 3.1 + RHIDE 1.4
//  Target      : DOS v5.0+


#include "pacgame.h"
#include "string.h"

DATAFILE *set_graphics[8];
DATAFILE *graphics;
DATAFILE *sprites;
DATAFILE *mapData;
DATAFILE *sounds;

BITMAP *screen_bitmap;
BITMAP *map_bitmap;

Button *btn_new_game;
Button *btn_map_edit;
Button *btn_exit;

int sound_on     = TRUE;
int music_on     = TRUE;
int sound_volume = 80;
int music_volume = 200;
int dif_level    = 1;


/* +----------------------------------------------------------- Error ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Mostra uma msg de erro e sai do jogo                     |
   +---------------------------------------------------------------------+ */

void Error(char *string)
{
  allegro_exit();                                // sai da biblioteca Allegro
  printf("Error: %s\n", string);
  exit(1);
}

/* +---------------------------------------------------- fim de Error ---+ */

/* +------------------------------------------------------- FileExist ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Verifica se o ficheiro indicado existe                   |
   +---------------------------------------------------------------------+ */

bool FileExist(char *filename)
{
  return file_exists(filename, FA_RDONLY | FA_ARCH, NULL);
}

/* +------------------------------------------------ fim de FileExist ---+ */

/* +------------------------------------------------------- DrawTitle ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Mostra o cran principal do jogo                         |
   +---------------------------------------------------------------------+ */

void DrawTitle(void)
{
  show_mouse(NULL);   
  fade_out(255);
  blit((BITMAP*)graphics[GTITLE].dat, screen, 0, 0, 0, 0, 640, 480);
  btn_new_game->DrawButton();
  btn_map_edit->DrawButton();
  btn_exit->DrawButton();
  show_mouse(NULL);
  fade_in((RGB*)graphics[PAL_TITLE2].dat, 5);
  show_mouse(screen);
}

/* +----------------------------------------------- fim de DrawTitle ---+ */

/* +-------------------------------------------------------- PlaySong ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Toca uma msica aleatriamente                           |
   +---------------------------------------------------------------------+ */

void PlaySong(void)
{
  if (music_on)
    play_midi((MIDI*)sounds[SONG1+Random(MAX_SONGS)].dat, TRUE);
}

/* +------------------------------------------------- fim de PlaySong ---+ */

/* +-------------------------------------------------------- do_begin ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Faz a introduo do jogo                                 |
   +---------------------------------------------------------------------+ */

void do_begin()
{
  play_midi((MIDI*)sounds[INTRO].dat, TRUE);
  show_mouse(NULL);      
  fade_out(255);
  blit((BITMAP*)graphics[BEGIN].dat, screen, 0, 0, 0, 0, 640, 480);
  show_mouse(NULL);
  fade_in((RGB*)graphics[PAL_BEGIN].dat, 5);
  // show_mouse(screen);

  do {
    if (key[KEY_ESC])
      break;
  } while(1==1);

  show_mouse(NULL);
  fade_out(255);
  blit((BITMAP*)graphics[TITLE].dat, screen, 0, 0, 0, 0, 640, 480);
  show_mouse(NULL);
  fade_in((RGB*)graphics[PAL_TITLE].dat, 5);
  // show_mouse(screen);

  do {
    if (key[KEY_ESC])
      break;
  } while(1==1);

}

/* +------------------------------------------------- fim de do_begin ---+ */

/* +------------------------------------------------------------ main ---+
   | Verso   : I (1.00.00)                                              |
   | Funo   : Controla todo o jogo                                     |
   +---------------------------------------------------------------------+ */

int main(int argc, char *argv[])
{
  FILE *file;

  printf("\nPac Man I\n");
  printf("Copyright (c) Cyrion Software\n\n");
  printf("Registered version!!\n\n");

  printf("Initializing game ...\n");
  srandom((int)time(NULL));
  allegro_init();                              // inicia a biblioteca Allegro
  install_mouse();
  install_timer();
  install_keyboard();
  initialise_joystick();

#ifdef __BEOS__
int counti;
char datapath[256];
for (counti=strlen(argv[0]) ; argv[0][counti]!='/' ; counti--);
strncpy(datapath, argv[0], counti);
chdir(datapath);
#endif

  printf("Installing game sound...\n");
  if (install_sound(DIGI_AUTODETECT, MIDI_AUTODETECT, NULL) != 0) {
    printf("No sound and music drivers found.\n");
    printf("Proceeding with no sound or music.\n");
    sound_on = FALSE;
    music_on = FALSE;
  }
  else {
    printf("Sound driver: %s %s\n", digi_driver->name, digi_driver->desc);
    printf("Music driver: %s %s\n", midi_driver->name, midi_driver->desc);
    set_volume(sound_volume, music_volume);
  }

  if (!FileExist("pacman.hsc")) {
    for (int i = 0; i < MAX_HIGHSCORES; i++) {
      HighScores[i].score = 0;
      HighScores[i].level = 0;
      strcpy(HighScores[i].name, "Pac Boy");
    }
  }
  else {
    file = fopen("pacman.hsc", "rb");
    if (!file)
      Error("Error reading file: pacman.hsc\nDelete it and try running Pac Man I again.\n");
    fread(&HighScores, sizeof(struct HighScoresStruct), MAX_HIGHSCORES, file);
    fclose(file);
  }
  if (!FileExist("pac.pm1"))    Error("Cannot find file pac.pm1!");
  if (!FileExist("pac2.pm1"))   Error("Cannot find file pac2.pm1!");
  if (!FileExist("maps.pm1"))   Error("Cannot find file maps.pm1!");
  if (!FileExist("set0.pm1"))   Error("Cannot find file set0.pm1!");
  if (!FileExist("set1.pm1"))   Error("Cannot find file set1.pm1!");
  if (!FileExist("set2.pm1"))   Error("Cannot find file set2.pm1!");
  if (!FileExist("set3.pm1"))   Error("Cannot find file set3.pm1!");
  if (!FileExist("set4.pm1"))   Error("Cannot find file set4.pm1!");
  if (!FileExist("set5.pm1"))   Error("Cannot find file set5.pm1!");
  if (!FileExist("set6.pm1"))   Error("Cannot find file set6.pm1!");
  if (!FileExist("set7.pm1"))   Error("Cannot find file set7.pm1!");
  if (!FileExist("sounds.pm1")) Error("Cannot find file sounds.pm1!");

  printf("Loading graphics ...\n");
  set_graphics[0] = load_datafile("set0.pm1");
  set_graphics[1] = load_datafile("set1.pm1");
  set_graphics[2] = load_datafile("set2.pm1");
  set_graphics[3] = load_datafile("set3.pm1");
  set_graphics[4] = load_datafile("set4.pm1");
  set_graphics[5] = load_datafile("set5.pm1");
  set_graphics[6] = load_datafile("set6.pm1");
  set_graphics[7] = load_datafile("set7.pm1");
  sprites = load_datafile("pac2.pm1");
  mapData = load_datafile("maps.pm1");
  graphics = load_datafile("pac.pm1");

  printf("Loading sounds ...\n");
  sounds = load_datafile("sounds.pm1");

  btn_new_game = new Button("GAME", 10, 230, 150, 255, 126, 173, (BITMAP*)graphics[BTN_GTITLE].dat);
  btn_map_edit = new Button("MAP EDITOR", 10, 260, 150, 285, 126, 173, (BITMAP*)graphics[BTN_GTITLE].dat);
  btn_exit = new Button("EXIT GAME", 10, 290, 150, 315, 126, 173, (BITMAP*)graphics[BTN_GTITLE].dat);

  set_gfx_mode(GFX_AUTODETECT, 640, 480, 0, 0);      // inicia o modo grfico

  map_bitmap = create_bitmap(441,441);
  screen_bitmap = create_bitmap(640, 480);
  do_begin();                                             // faz a introduo
  PlaySong();
  DrawTitle();           
  do {
    if (key[KEY_F10]) {
      RGB title_pal[256];
      get_palette(title_pal);
      BITMAP *tbmp = create_sub_bitmap(screen, 0, 0, SCREEN_W, SCREEN_H);
      save_pcx("editor.pcx", tbmp, title_pal);
      destroy_bitmap(tbmp);
    }
    if (btn_new_game->Clicked()) {
      stop_midi();
      StartGameMenu();
      DrawTitle();
      PlaySong();
    }
    else
      if (btn_map_edit->Clicked()) {
        stop_midi();
        main_edit();
        DrawTitle();
        PlaySong();
      }
      else
        if (btn_exit->Clicked())
          break;
        else
          if (key[KEY_ESC])
            break;
  } while (1==1);
  stop_midi();
  fade_out(2);
  file = fopen("pacman.hsc", "wb");
  if (file) {
    fwrite(&HighScores, sizeof(struct HighScoresStruct), MAX_HIGHSCORES, file);
    fclose(file);
  }
  allegro_exit();                               // fecha a biblioteca Allegro
  printf("\nPac Man I\n");
  printf("Copyright (c) Cyrion Software\n\n");
  printf("Registered version!!\n\n");

  return 0;
}